
// 
// Copyright (c) 2011 Jeff Flinn
// 
// Distributed under the Boost Software License, Version 1.0. (See accompanying 
// file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt) 
// 

#ifndef BOOST_PROCESS_POSIX_INITIALIZER_ENVIRONMENT_HPP
#define BOOST_PROCESS_POSIX_INITIALIZER_ENVIRONMENT_HPP

namespace boost { namespace process { namespace posix {

    struct executor;

    struct environment : public initializer 
    {
        environment(const env* e) : m_vars(from(e)), m_var_ptrs() {}
        environment(            ) : m_vars(       ), m_var_ptrs() {}
        
        environment& add(const std::string& evar) { m_vars.push_back(evar); }
        
        void pre_fork_parent(executor& e)
        {
            m_var_ptrs.clear();
            m_var_ptrs.resize(m_vars.size() + 1);
            
            std::transform(m_vars.begin(), m_vars.end(), m_var_ptrs.begin(), boost::bind(&std::string::c_str, _1));
            
            e.m_env_vars_ptrs = &m_var_ptrs[0];
        }
        
        static vars_t from(const env*)
        {
            return vars_t();
        }

        typedef std::vector<std::string> vars_t;
        typedef std::vector<const char*> var_ptrs_t;
        
        vars_t     m_vars;
        var_ptrs_t m_var_ptrs;
    };

}}}

#endif
