
// 
// Copyright (c) 2011 Jeff Flinn
// 
// Distributed under the Boost Software License, Version 1.0. (See accompanying 
// file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt) 
// 

#ifndef BOOST_PROCESS_POSIX_INITIALIZER_HPP
#define BOOST_PROCESS_POSIX_INITIALIZER_HPP

#include <boost/process/config.hpp>

namespace boost { namespace process { namespace posix {

struct executor;

struct initializer 
{
    void    pre_fork_parent(executor&) const {}
    void   post_fork_parent(executor&) const {}
    void failed_fork_parent(executor&) const {}
    void   post_fork_child (executor&) const {}
    void failed_exec_child (executor&) const {}
    
    struct lazy
    {
        struct pre_fork_parent
        {
            executor& m_e;
        
            pre_fork_parent(executor& e) : m_e(e) {}
            
            template<class I> void operator()(const I& i) const { return i.pre_fork_parent(m_e); }
        };
        struct post_fork_parent
        {
            executor& m_e;
        
            post_fork_parent(executor& e) : m_e(e) {}
            
            template<class I> void operator()(const I& i) const { return i.post_fork_parent(m_e); }
        };
        struct failed_fork_parent
        {
            executor& m_e;
        
            failed_fork_parent(executor& e) : m_e(e) {}
            
            template<class I> void operator()(const I& i) const { return i.failed_fork_parent(m_e); }
        };
        struct post_fork_child
        {
            executor& m_e;
        
            post_fork_child(executor& e) : m_e(e) {}
            
            template<class I> void operator()(const I& i) const { return i.post_fork_child(m_e); }
        };
        struct failed_exec_child
        {
            executor& m_e;
        
            failed_exec_child(executor& e) : m_e(e) {}
            
            template<class I> void operator()(const I& i) const { return i.failed_exec_child(m_e); }
        };
    };
};

}}}

#endif
