
// 
// Copyright (c) 2011 Jeff Flinn
// 
// Distributed under the Boost Software License, Version 1.0. (See accompanying 
// file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt) 
// 

#ifndef BOOST_PROCESS_POSIX_INITIALIZER_THROW_ON_LAUNCH_ERROR_HPP
#define BOOST_PROCESS_POSIX_INITIALIZER_THROW_ON_LAUNCH_ERROR_HPP

namespace boost { namespace process { namespace posix {

    struct executor;

    struct throw_on_launch_error : public initializer 
    {
        typedef int file_descriptor;
        
        mutable file_descriptor  m_read_end;
        mutable file_descriptor m_write_end;
        mutable int                 m_errno;
        
        throw_on_launch_error()
        :  m_read_end(-1)
        , m_write_end(-1)
        ,     m_errno( 0)
        {}
        
        ~throw_on_launch_error()
        {
            if( m_read_end!=-1) close( m_read_end);
            if(m_write_end!=-1) close(m_write_end);
        }
        
        void pre_fork_parent(executor&) const
        {
            {
                int pipe_fds[2] = {-1, -1};

                pipe(pipe_fds);

                 m_read_end = ensure_user_fd_index(pipe_fds[0]);
                m_write_end = ensure_user_fd_index(pipe_fds[1]);
            }
            
            signal(SIGPIPE, SIG_IGN); // Who's to say this doesn't change elsewhere

            fcntl( m_read_end, F_SETFD, FD_CLOEXEC|fcntl( m_read_end, F_GETFD, 0)); 
            fcntl(m_write_end, F_SETFD, FD_CLOEXEC|fcntl(m_write_end, F_GETFD, 0)); 
        }
        void post_fork_parent(executor&) const
        {
            close(m_write_end); m_write_end = -1;
            
            if(read(m_read_end, &m_errno, sizeof(m_errno)))
            {
                close(m_read_end);
            
                throw_if(m_errno);
            }
            close(m_read_end);
        }
        void failed_fork_parent(executor&) const
        {
            throw_if(errno); //@TODO: should errno be an executor member set by executor to ensure it's the proper one
        }
        void post_fork_child(executor&) const
        {
            close(m_read_end); m_read_end = -1;
        }
        void failed_exec_child(executor&) const
        {
            m_errno = errno;
        
            write(m_write_end, &m_errno, sizeof(m_errno));    
        }
        
        static void throw_if(int err) { /*if(err) throw boost::system::system_error(err);*/ }

        static int ensure_user_fd_index(int fd_index)
        {
            //BOOST_STATIC_ASSERT( STDIN_FILENO == 0)
            //BOOST_STATIC_ASSERT(STDOUT_FILENO == 1)
            //BOOST_STATIC_ASSERT(STDERR_FILENO == 2)

            const int user_fileno_begin = 3;

            int user_fd_index = fcntl(fd_index, F_DUPFD, user_fileno_begin);

            close(fd_index);

            return user_fd_index;
        }
    };

}}}

#endif // BOOST_PROCESS_POSIX_INITIALIZER_THROW_ON_LAUNCH_ERROR_HPP
